/*
 *      Copyright(C) 2004 NAKAYAMA Hiroyoshi
 *
 * 	Licensed under the terms of the GNU GPL License version 2.
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/io.h>

#include "devmsr.h"

/* #define DEBUG */

int rddevmsr(int cpu, unsigned long index, unsigned long long *val)
{
  char cpuname[16];
  int fh;
  off_t   r_lseek;
  ssize_t r_read;
  
  snprintf (cpuname, 15,"/dev/cpu/%d/msr", cpu);
  fh = open (cpuname, O_RDONLY);
#ifdef DEBUG
  printf("[ rdmsr  start ]\n%s (fh = %d)\n",cpuname, fh);
#endif
  
  if ( fh == -1 ) {
    printf("rddevmsr:Not such a device(%s).\n",cpuname );
    return(-1);
  }

  r_lseek = lseek (fh, index, SEEK_SET);
  r_read  = (read (fh, val, 8) == 8);
#ifdef DEBUG
  {
    unsigned long l,h;
    l = *val;
    h = (*val)>>32;
    printf("lseek %08X, read %d\n",r_lseek, r_read);
    printf("index=%08X , val=%08X_%08X\n",index,h,l);
  }
#endif
  close (fh);
  
#ifdef DEBUG
  printf("[ rdmsr  end (%d)]\n",r);
#endif

  return(r_read);
}

int wrdevmsr(int cpu, unsigned long index, unsigned long long *val)
{
  int fh;
  char cpuname[16];
  int r;

  snprintf (cpuname,15, "/dev/cpu/%d/msr", cpu);
  fh = open (cpuname, O_WRONLY);
#ifdef DEBUG
  {
    unsigned long l,h;

    printf("func:wrmsr()\n");
    printf(" open device \"%s\" at O_WRONLY\n",cpuname);
    l = *val;
    h = (*val)>>32;
    printf(" seek to %0xh, write value:%08x_%08xh(64bits)\n",index,h,l);
  }
#endif

  if (fh==-1) {
    printf("wrdevmsr:Not such a device(%s).\n",cpuname );
    return(-1);
  }

  lseek (fh, index, SEEK_SET);
  r = (write (fh, val, 8) == 8);
  close (fh);

  return (r);
}
