/*
 *	This file is part of tkK7mauto
 *      Copyright(C) 2004 NAKAYAMA Hiroyoshi
 *
 * 	Licensed under the terms of the GNU GPL License version 2.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "miscio.h"

#define CPUSTATEFILE "/usr/local/var/cpustate/state"

static float FID_DATA[32] = {
  11.0, 11.5, 12.0, 12.5,  5.0,  5.5,  6.0,  6.5,
   7.0,  7.5,  8.0,  8.5,  9.0,  9.5, 10.0, 10.5,
   3.0, 19.0,  4.0, 20.0, 13.0, 13.5, 14.0, 21.0,
  15.0, 22.0, 16.0, 16.5, 17.0, 18.0, 23.0, 24.0,
};
unsigned char FIDsort_DATA[32] = {
  0x10, 0x12, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C,
  0x0D, 0x0E, 0x0F, 0x00, 0x01, 0x02, 0x03, 0x14, 0x15, 0x16, 0x18,
  0x1A, 0x1B, 0x1C, 0x1D, 0x11, 0x13, 0x17, 0x19, 0x1E, 0x1F,
};

unsigned char getOptimumFID(float mhz, float fsb)
{
  float m;
  int i;
  unsigned char fid;

  m = (float)mhz / fsb;

  fid = FIDsort_DATA[0];
  if (m < FID_DATA[FIDsort_DATA[0]])
    return(fid);

  for (i = 1; i < 32; i ++) {
    if ( FID_DATA[FIDsort_DATA[i]] > m) {
      fid = FIDsort_DATA[i-1];
      break;
    }
  }
  if (i > 31)
    fid = FIDsort_DATA[31];

  return(fid);
}

#define MAX_FSB 200
#define MIN_FSB 100
float getFSB(void)
{
  FILE *fp;
  char buf[64];
  float fsb;

  fsb = 0;
  if ((fp = fopen(CPUSTATEFILE, "ro")) != NULL) {
    while(GetWord(fp, buf)) {
      //      printf("[%s]\n",buf);
      if (strstr(buf,"FSB") == buf) {
	if (GetWord(fp,buf)) {
	  //	  printf("[**%s**]\n",buf);
	  fsb = atof(buf);
	}
      }
    }
    (void)fclose(fp);
  }

  if (fsb == 0)
    fsb = MAX_FSB;
  return(fsb);
}

float makeFIDtable(int n, float max_f, unsigned char *FIDtable)
{
  int i;
  float dif;
  int  min,max;
  int  fid;
  float  fsb;

  fsb = (int)(getFSB() * 10);
  fsb /= 10.0;

  min = FID_DATA[getOptimumFID(300,fsb)]*fsb;
  max = max_f;

  dif = max-min;
  dif /= (float)(n-1);

//  printf("%d %d %2.1f\n",min,max,dif);
  for (i = 0; i < 8; i ++) {
    //    printf("[%f]\n",(float)min+dif*(float)i);
    fid = getOptimumFID((float)min+dif*(float)i,fsb);
    FIDtable[i] = fid;
    //    printf("%02X(%2.1f...%3.2f)\n",fid,FID_DATA[fid],FID_DATA[fid]*fsb);
  }

  return(fsb);
}


//#define DEBUG
#ifdef DEBUG
int main(int argc, char **argv)
{
  int i,n;
  float fsb;
  float mhz;
  unsigned char FIDtable[8];

  n = 8;
  fsb = makeFIDtable(n, 2000, FIDtable);
  printf("FSB:%3.2f\n",fsb);
  for (i = 0; i < n; i ++) { 
      printf("%02X(%2.1f...%3.2f)\n",
	     FIDtable[i],
	     FID_DATA[FIDtable[i]],
	     FID_DATA[FIDtable[i]]*fsb);
  }
}
#endif

