/*
 *	This file is part of tkK7mauto
 *      Copyright(C) 2004 NAKAYAMA Hiroyoshi
 *
 * 	Licensed under the terms of the GNU GPL License version 2.
 */
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <linux/version.h>
#include <math.h>

#include "sensors.h"

/* #define DEBUG */

#ifndef NO_SENSOR

#if 0
#define VCORE   "/sys/devices/platform/i2c-0/0-0290/in_input0"
#define SYSTEMP "/sys/devices/platform/i2c-0/0-0290/temp_input1"
#define CPUTEMP "/sys/devices/platform/i2c-0/0-0290/temp_input2"
#define SYSFAN  "/sys/devices/platform/i2c-0/0-0290/fan_input1"
#define CPUFAN  "/sys/devices/platform/i2c-0/0-0290/fan_input2"
#else
#define VCORE   "/sys/devices/platform/i2c-0/0-0290/in0_input"
#define SYSTEMP "/sys/devices/platform/i2c-0/0-0290/temp1_input"
#define CPUTEMP "/sys/devices/platform/i2c-0/0-0290/temp2_input"
#define SYSFAN  "/sys/devices/platform/i2c-0/0-0290/fan1_input"
#define CPUFAN  "/sys/devices/platform/i2c-0/0-0290/fan2_input"
#endif

int prev_user, prev_nice, prev_system, prev_idle;

float SensorGetVcore(void)
{
  static int pstat = -1;
  char buf[1024];
  int len;
  int data;

  if (pstat == -1) {
    pstat = open(VCORE , O_RDONLY);
  }

  if(pstat == -1) {
    perror(VCORE);
    exit(1);
  }
  lseek(pstat, 0, SEEK_SET);
  len = read(pstat, buf, sizeof(buf));

  buf[len] = 0;
  sscanf(buf,"%d", &data);

  return((float)data/1000.0);
}

float SensorGetSYStemp(void)
{
  static int pstat = -1;
  char buf[1024];
  int len;
  int data;

  if (pstat == -1) {
    pstat = open(SYSTEMP , O_RDONLY);
  }

  if(pstat == -1) {
    perror(SYSTEMP);
    exit(1);
  }
  lseek(pstat, 0, SEEK_SET);
  len = read(pstat, buf, sizeof(buf));

  buf[len] = 0;
  sscanf(buf,"%d", &data);

  return((float)data/1000.0);
}

float SensorGetCPUtemp(void)
{
  static int pstat = -1;
  char buf[1024];
  int len;
  int data;

  if (pstat == -1) {
    pstat = open(CPUTEMP , O_RDONLY);
  }

  if(pstat == -1) {
    perror(CPUTEMP);
    exit(1);
  }
  lseek(pstat, 0, SEEK_SET);
  len = read(pstat, buf, sizeof(buf));

  buf[len] = 0;
  sscanf(buf,"%d", &data);

  return((float)data/1000.0);
}

float SensorGetSYSfan(void)
{
  static int pstat = -1;
  char buf[1024];
  int len;
  int data;

  if (pstat == -1) {
    pstat = open(SYSFAN , O_RDONLY);
  }

  if(pstat == -1) {
    perror(SYSFAN);
    exit(1);
  }
  lseek(pstat, 0, SEEK_SET);
  len = read(pstat, buf, sizeof(buf));

  buf[len] = 0;
  sscanf(buf,"%d", &data);

  return((float)data);
}

float SensorGetCPUfan(void)
{
  static int pstat = -1;
  char buf[1024];
  int len;
  int data;

  if (pstat == -1) {
    pstat = open(CPUFAN , O_RDONLY);
  }

  if(pstat == -1) {
    perror(CPUFAN);
    exit(1);
  }
  lseek(pstat, 0, SEEK_SET);
  len = read(pstat, buf, sizeof(buf));

  buf[len] = 0;
  sscanf(buf,"%d", &data);

  return((float)data);
}


#ifdef DEBUG
int main ()
{
  float f;

  while (1) {
    f = SensorGetVcore();
    printf("Vc:%f\n",f);
    f = SensorGetSYStemp();
    printf("SysT:%f\n",f);
    f = SensorGetCPUtemp();
    printf("CpuT:%f\n",f);
    f = SensorGetSYSfan();
    printf("Sysf:%f\n",f);
    f = SensorGetCPUfan();
    printf("Cpuf:%f\n",f);
    sleep (1);
  }
}
#endif

#endif /* NO_SENSOR */
